// *********************************************************************
// THINGS TO TRY IN C25SIM
// *********************************************************************

// *********************************************************************
// Execute one or more instructions, and report on registers and flags
// *********************************************************************

: s       step long_report ;
: steps   0 do step loop long_report ;

// *********************************************************************
// FOO.C exercises the C25's bit reversal addressing mode. Whenever
// AR5 changes, "bit_rvs" displays the values of AR4 and AR5.
// *********************************************************************

var previous
: change?   dup previous @ = if drop 0 else previous ! 1 then ;

: 1+!   dup @ 1 + swap ! ;  // an ingenious invention of Forth hackers

var ar5_changes

: bit_rvs
-1 previous !
0 ar5_changes !
0
begin
   step
   ar5 @ change? if
      ar4 @ .h ar5 @ .h cr
      ar5_changes 1+!
   then
   ar5_changes @ 16 = if drop 1 then
dup until
drop report ;

// *********************************************************************
// "go-til" takes an address, and executes instructions til it gets there.
// *********************************************************************

: key-pressed  key -1 = not ;

: wait-for-key-press  begin key-pressed until ;

: go-til
begin
   step
   dup pc @ = if drop -1 then  // halt when pc equals input
   key-pressed if drop -1 then  // or halt if key is pressed
dup -1 = until
drop long_report ;   // when you halt, lose input and report c25 status

// *********************************************************************
// An example of implementing wait states for program memory.
// *********************************************************************

// range: (addr addrlo addrhi) --> (flag)
// flag is 1 if addrlo <= addr <= addrhi, 0 otherwise (compares unsigned)
: range   3 pick u< not rot rot u< not & ;

// a teeny island of really slow program memory
: pmem_waits  0x95 0xa0 range if 5 else 0 then ;

// *********************************************************************
// Playing with breakpoints
// *********************************************************************

_main +bp 0xAA +bp  // set some breakpoints

: breaks  0 do go_til_bp loop report ;  // pass N breakpoints, report
: break   1 breaks ;  // go to next breakpoint, report

// *********************************************************************
// Reading the C25's hardware stack
// *********************************************************************

: top?   sp @ dup 0= if ." ? " drop
         else 1 - c25stack @ . then ;

: try_top
   begin
      step
      top?
   key-pressed until
   cr ;
